@@ -3,7 +3,7 @@
  *  Implement the DUMPVCP and LOADVCP commands
  */
 
-// Copyright (C) 2014-2021 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2022 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #ifndef APP_DUMPLOAD_H_
@@ -12,10 +12,13 @@
 #include <base/displays.h>
 #include <base/status_code_mgt.h>
 
-bool loadvcp_by_file(const char * fn, Display_Handle * dh);
+Status_Errno_DDC
+app_loadvcp_by_file(const char * fn, Display_Handle * dh);
 
-Status_Errno_DDC dumpvcp_as_file(Display_Handle * dh, const char * optional_filename);
+Status_Errno_DDC
+app_dumpvcp_as_file(Display_Handle * dh, const char * optional_filename);
 
-void init_app_dumpload();
+void
+init_app_dumpload();
 
 #endif /* APP_DUMPLOAD_H_ */
