@@ -1,37 +1,44 @@
-/** \file app_getvcp.h
+/** @file app_getvcp.h
+ *  Implement the GETVCP command
  */
 
- //Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+ //Copyright (C) 2014-2022 Sanford Rockowitz <rockowitz@minsoft.com>
  // SPDX-License-Identifier: GPL-2.0-or-later
 
 #ifndef APP_GETVCP_H_
 #define APP_GETVCP_H_
 
+#include "public/ddcutil_types.h"
+
 /** \cond */
 #include <stdbool.h>
 /** \endcond */
 
-#include "util/error_info.h"
 #include "base/displays.h"
-#include "base/feature_sets.h"
+#include "base/feature_set_ref.h"
 #include "base/status_code_mgt.h"
+
 #include "cmdline/parsed_cmd.h"
 
-Public_Status_Code
+Status_Errno_DDC
+app_show_single_vcp_value_by_feature_id(
+      Display_Handle *      dh,
+      DDCA_Vcp_Feature_Code feature_id,
+      bool                  force);
+
+Status_Errno_DDC
 app_show_vcp_subset_values_by_dh(
       Display_Handle *      dh,
       VCP_Feature_Subset    subset,
       Feature_Set_Flags     flags,
-      Byte_Bit_Flags        features_seen);
+      Bit_Set_256 *         features_seen);
 
-Public_Status_Code
+Status_Errno_DDC
 app_show_feature_set_values_by_dh(
       Display_Handle *      dh,
       Parsed_Cmd *          parsed_cmd);
 
 void
-app_read_changes_forever(
-      Display_Handle *      dh,
-      bool                  force_no_fifo);
+init_app_getvcp();
 
 #endif /* APP_GETVCP_H_ */
