@@ -3,7 +3,7 @@
  *  Implement VCPINFO and (deprecated) LISTVCP commands
  */
 
-// Copyright (C) 2020-2023 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2020-2025 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #include "config.h"
@@ -20,10 +20,11 @@
 #include "base/rtti.h"
 #include "base/vcp_version.h"
 
-#include "vcp/vcp_feature_set.h"
+#include "dynvcp/dyn_feature_set.h"
+#include "dynvcp/vcp_feature_set.h"
 #include "vcp/vcp_feature_codes.h"
 
-#include "app_ddcutil/app_vcpinfo.h"
+#include "app_vcpinfo.h"
 
 
 /** Creates humanly readable interpretation of VCP feature flags.
@@ -236,6 +237,8 @@ interpret_ddca_version_feature_flags_type(
       result = "Continuous (complex)";
    else if (feature_flags & DDCA_SIMPLE_NC)
       result = "Non-Continuous (simple)";
+   else if (feature_flags & DDCA_EXTENDED_NC)
+      result = "Non-Continuous (extended)";
    else if (feature_flags & DDCA_COMPLEX_NC)
       result = "Non-Continuous (complex)";
    else if (feature_flags & DDCA_NC_CONT)
@@ -373,29 +376,52 @@ app_vcpinfo(Parsed_Cmd * parsed_cmd)
       fsflags |= FSF_RO_ONLY;
    if (parsed_cmd->flags & CMD_FLAG_WO_ONLY)
       fsflags |= FSF_WO_ONLY;
+   // if (parsed_cmd->flags & CMD_FLAG_ENABLE_UDF)    // Do I want
+   //    fsflags |= FSF_CHECK_UDF;
 
+   Dyn_Feature_Set * fset = create_dyn_feature_set_from_feature_set_ref(
+                               parsed_cmd->fref,
+                               parsed_cmd->mccs_vspec,
+                               fsflags);
+#ifdef OLD
    VCP_Feature_Set * fset = create_vcp_feature_set_from_feature_set_ref(
                                parsed_cmd->fref,
                                parsed_cmd->mccs_vspec,
                                fsflags);
+#endif
    if (IS_DBGTRC(debug, (DDCA_TRC_TOP | DDCA_TRC_VCP)) )
-      dbgrpt_vcp_feature_set(fset, 2);
+      dbgrpt_dyn_feature_set(fset, /*verbose=*/false, 2);
+
 
    if (!fset) {
       vcpinfo_ok = false;
    }
    else {
+      bool saved_prefix_report_output = rpt_set_ornamentation_enabled(false);
+
       if ( get_output_level() <= DDCA_OL_TERSE)
-         report_vcp_feature_set(fset, 0);
+         report_dyn_feature_set(fset, 0);
       else {
-         int ct =  get_vcp_feature_set_size(fset);
+#ifdef OLD
+         int ct = get_vcp_feature_set_size(fset);
+         for (int ndx = 0; ndx < ct; ndx++) {
+             VCP_Feature_Table_Entry * pentry = get_vcp_feature_set_entry(fset, ndx);
+             report_vcp_feature_table_entry(pentry, 0);
+         }
+#endif
+         int ct =  dyn_get_feature_set_size(fset);
          int ndx = 0;
          for (;ndx < ct; ndx++) {
-            VCP_Feature_Table_Entry * pentry = get_vcp_feature_set_entry(fset, ndx);
+            Display_Feature_Metadata * dfm = g_ptr_array_index(fset->members_dfm, ndx);
+            // VCP_Feature_Table_Entry * pentry = get_vcp_feature_set_entry(fset, ndx);
+            VCP_Feature_Table_Entry * pentry = vcp_find_feature_by_hexid(dfm->feature_code);
+            assert(pentry);   // every possible feature code has a feature table entry
             report_vcp_feature_table_entry(pentry, 0);
          }
       }
-      free_vcp_feature_set(fset);
+      rpt_set_ornamentation_enabled(saved_prefix_report_output);
+      // free_vcp_feature_set(fset);
+      free_dyn_feature_set(fset);
    }
 
    DBGTRC_RET_BOOL(debug, DDCA_TRC_VCP|DDCA_TRC_TOP, vcpinfo_ok, "");
