@@ -1,6 +1,4 @@
-/** @file displays.h
- * Display Specification
- */
+/** @file displays.h  Display Specification  */
 
 // Copyright (C) 2014-2024 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
@@ -19,11 +17,18 @@
 #include "public/ddcutil_types.h"
 
 #include "core.h"
+#include "ddcutil_types_internal.h"
 #include "dynamic_features.h"
 #include "feature_set_ref.h"
 #include "monitor_model_key.h"
 #include "vcp_version.h"
 
+extern bool      terminate_watch_thread;
+
+extern GPtrArray * all_display_refs;         // all detected displays, array of Display_Ref *
+extern GMutex      all_display_refs_mutex;
+extern bool        debug_locks;
+
 
 /** \file
 Display Specification
@@ -47,6 +52,7 @@ For I2C displays, the device must be opened.  Display_Handle then contains the o
 // *** Initialization ***
 
 void init_displays();
+void terminate_displays();
 
 
 // *** DDCA_IO_Path ***
@@ -59,6 +65,7 @@ char *  dpath_short_name_t(DDCA_IO_Path * dpath);
 char *  dpath_repr_t(DDCA_IO_Path * dpath);  // value valid until next call
 int     dpath_hash(DDCA_IO_Path path);
 DDCA_IO_Path i2c_io_path(int busno);
+DDCA_IO_Path usb_io_path(int hiddev_devno);
 
 
 // *** Display_Identifier ***
@@ -155,6 +162,7 @@ typedef uint16_t Dref_Flags;
 #define DREF_OPEN                                      0x0800
 #define DREF_DDC_BUSY                                  0x1000
 #define DREF_REMOVED                                   0x2000
+#define DREF_DDC_DISABLED                              0x4000
 #define DREF_DPMS_SUSPEND_STANDBY_OFF                  0x8000
 
 char * interpret_dref_flags_t(Dref_Flags flags);
@@ -165,6 +173,7 @@ char * interpret_dref_flags_t(Dref_Flags flags);
 #define DISPNO_PHANTOM -2
 #define DISPNO_REMOVED -3
 #define DISPNO_BUSY    -4
+#define DISPNO_DDC_DISABLED -5
 
 #define DISPLAY_REF_MARKER "DREF"
 /** A **Display_Ref** is a logical display identifier.
@@ -172,6 +181,7 @@ char * interpret_dref_flags_t(Dref_Flags flags);
  */
 typedef struct _display_ref {
    char                     marker[4];
+   uint                     dref_id;
    DDCA_IO_Path             io_path;
    int                      usb_bus;
    int                      usb_device;
@@ -188,12 +198,27 @@ typedef struct _display_ref {
    uint64_t                 next_i2c_io_after;     // nanosec
    struct _display_ref *    actual_display;        // if dispno == -2
    DDCA_IO_Path *           actual_display_path;   // alt to actual_display
+#ifdef OLD
    char *                   driver_name;           //
+#endif
    struct Per_Display_Data* pdd;
    char *                   drm_connector;         // e.g. card0-HDMI-A-1  // REDUNDANT - IDENTICAL TO Bus_Info.drm_connector
+   int                      drm_connector_id;      // identical to Bus_Info.drm_connector_id
    char *                   communication_error_summary;
+   uint64_t                 creation_timestamp;
+   GMutex                   access_mutex;
 } Display_Ref;
 
+
+void dbgrpt_published_dref_hash(const char * msg, int depth);
+void init_published_dref_hash();
+void reset_published_dref_hash();
+void add_published_dref_id_by_dref(Display_Ref * dref);
+Display_Ref *    dref_from_published_ddca_dref(DDCA_Display_Ref ddca_dref);
+DDCA_Display_Ref dref_to_ddca_dref(Display_Ref * dref);
+
+
+#define DREF_BUSNO(_dref) ((_dref)->io_path.path.i2c_busno)
 #define ASSERT_DREF_IO_MODE(_dref, _mode)  \
    assert(_dref && \
           memcmp(dref->marker, DISPLAY_REF_MARKER, 4) == 0) && \
@@ -202,20 +227,34 @@ typedef struct _display_ref {
 Display_Ref * create_base_display_ref(DDCA_IO_Path io_path);
 Display_Ref * create_bus_display_ref(int busno);
 Display_Ref * create_usb_display_ref(int bus, int device, char * hiddev_devname);
-void          dbgrpt_display_ref(Display_Ref * dref, int depth);
+void          dbgrpt_display_ref(Display_Ref * dref, bool include_businfo, int depth);
+void          dbgrpt_display_ref0(Display_Ref * dref, int depth);
+void          dbgrpt_display_ref_summary(Display_Ref * dref, bool include_businfo, int depth);
 char *        dref_short_name_t(Display_Ref * dref);
 char *        dref_repr_t(Display_Ref * dref);  // value valid until next call
+char *        dref_reprx_t(Display_Ref * dref);  // value valid until next call
+char *        ddci_dref_repr_t(DDCA_Display_Ref * ddca_dref);  // value valid until next call
 DDCA_Status   free_display_ref(Display_Ref * dref);
 Display_Ref * copy_display_ref(Display_Ref * dref);
+void          dref_lock(Display_Ref * dref);
+void          dref_unlock(Display_Ref * dref);
 
 // Do two Display_Ref's identify the same device?
-bool dref_eq(Display_Ref* this, Display_Ref* that);
+bool          dref_eq(Display_Ref* this, Display_Ref* that);
+
+const char *  dref_get_i2c_driver(Display_Ref* dref);
 
 #ifdef UNUSED
 bool dref_set_alive(Display_Ref * dref, bool alive);
 bool dref_get_alive(Display_Ref * dref);
 #endif
 
+Display_Ref* get_dref_by_busno_or_connector(int busno, const char * connector, bool ignore_invalid);
+#define      GET_DREF_BY_BUSNO(_busno, _ignore) \
+             get_dref_by_busno_or_connector(_busno,NULL, (_ignore))
+#define      GET_DREF_BY_CONNECTOR(_connector_name, _ignore_invalid) \
+             get_dref_by_busno_or_connector(-1, _connector_name, _ignore_invalid)
+
 // *** Display_Handle ***
 
 #define DISPLAY_HANDLE_MARKER "DSPH"
@@ -225,12 +264,14 @@ typedef struct {
    Display_Ref* dref;
    int          fd;     // file descriptor
    char *       repr;
+   char *       repr_p;
    bool         testing_unsupported_feature_active;
 } Display_Handle;
 
 Display_Handle * create_base_display_handle(int fd, Display_Ref * dref);
 void             dbgrpt_display_handle(Display_Handle * dh, const char * msg, int depth);
 char *           dh_repr(Display_Handle * dh);
+char *           dh_repr_p(Display_Handle * dh);
 void             free_display_handle(Display_Handle * dh);
 
 // For internal display selection functions
@@ -259,5 +300,21 @@ typedef struct {
    char * detail;
 } Bus_Open_Error;
 
+void free_bus_open_error(Bus_Open_Error * boe);
+
+typedef enum {
+   Watch_Mode_Dynamic,
+   Watch_Mode_Poll,
+   Watch_Mode_Xevent,
+   Watch_Mode_Udev,
+} DDC_Watch_Mode;
+
+const char * watch_mode_name(DDC_Watch_Mode mode);
+
+bool add_disabled_display(Monitor_Model_Key * mmk);
+bool add_disabled_mmk_by_string(const char * mmid);
+void dbgrpt_ddc_disabled_table(int depth);
+bool is_disabled_mmk(Monitor_Model_Key mmk);
+
 
 #endif /* DISPLAYS_H_ */
