@@ -1,8 +1,9 @@
-/** \file linux_errno.c
+/** @file linux_errno.c
+ *
  * Linux errno descriptions
  */
 
-// Copyright (C) 2014-2022 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2023 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #include "config.h"
@@ -15,6 +16,7 @@
 #include <string.h>
 /** \endcond */
 
+#include "util/debug_util.h"
 #include "util/string_util.h"
 
 #include "base/linux_errno.h"
@@ -388,7 +390,7 @@ Status_Code_Info * find_errno_description(int errnum) {
       // char * desc = strerror(errnum);
       // result->description  = malloc(strlen(desc)+1);
       // strcpy(result->description, desc);
-      result->description = strdup(strerror(errnum));
+      result->description = g_strdup(strerror(errnum));
    }
    if (debug)
       printf("(%s) Returning %p\n", __func__, (void*)result);
@@ -438,15 +440,19 @@ Status_Code_Info * get_negative_errno_info(int errnum) {
  * @return  true if found, false if not
  */
 bool errno_name_to_number(const char * errno_name, int * perrno) {
+   bool debug = false;
+   DBGF(debug, "Starting. errno_name=%s", errno_name);
    int found = false;
    *perrno = 0;
    for (int ndx = 0; ndx < errno_desc_ct; ndx++) {
+      DBGF(debug, "Comparing: errno_desc[ndx] = %s", errno_desc[ndx].name);
        if ( streq(errno_desc[ndx].name, errno_name) ) {
           *perrno = -errno_desc[ndx].code;
           found = true;
           break;
        }
    }
+   DBGF(debug, "Done.   Returning: %s. *perrno = %d", sbool(found), *perrno);
    return found;
 }
 
