@@ -6,7 +6,7 @@
  * tracing and and maintain sleep statistics.
  */
 
-// Copyright (C) 2014-2022 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2025 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #ifndef BASE_SLEEP_H_
@@ -39,4 +39,16 @@ void         init_sleep_stats();
 Sleep_Stats  get_sleep_stats();
 void         report_sleep_stats(int depth);
 
+// Perform display watch related sleeps
+
+#define DW_SLEEP_MILLIS(_millis, _msg) \
+   do { \
+      dw_sleep_millis(DDCA_SYSLOG_NOTICE, __func__, __LINE__, __FILE__, _millis, _msg); \
+   } while(0)
+#define DW_SLEEP_MILLIS2(_syslog_level, _millis, _msg) \
+   do { \
+      dw_sleep_millis(_syslog_level, __func__, __LINE__, __FILE__, _millis, _msg); \
+   } while(0)
+void dw_sleep_millis(DDCA_Syslog_Level level, const char * func, int line, const char * file, uint millis, const char * msg);
+
 #endif /* BASE_SLEEP_H_ */
