@@ -1,6 +1,6 @@
 /** @file ddc_display_selection.c */
 
-// Copyright (C) 2022-2023 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2022-2025 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #include "config.h"
@@ -162,6 +162,12 @@ bye:
 }
 
 
+/** Finds the first display reference satisfying a set of display criteria.
+ *  Phantom displays are ignored.
+ *
+ *  @param  criteria identifiers to check
+ *  @return display reference if found, NULL if not
+ */
 static Display_Ref *
 ddc_find_display_ref_by_criteria(Display_Criteria * criteria) {
    Display_Ref * result = NULL;
@@ -170,8 +176,11 @@ ddc_find_display_ref_by_criteria(Display_Criteria * criteria) {
       Display_Ref * drec = g_ptr_array_index(all_displays, ndx);
       TRACED_ASSERT(memcmp(drec->marker, DISPLAY_REF_MARKER, 4) == 0);
       if (ddc_test_display_ref_criteria(drec, criteria)) {
-         result = drec;
-         break;
+         // Ignore the match if it's a phantom display
+         if (drec->dispno != DISPNO_PHANTOM) {
+            result = drec;
+            break;
+         }
       }
    }
    return result;
@@ -179,7 +188,8 @@ ddc_find_display_ref_by_criteria(Display_Criteria * criteria) {
 
 
 /** Searches the master display list for a display matching the
- *  specified #Display_Identifier, returning its #Display_Ref
+ *  specified #Display_Identifier, returning its #Display_Ref.
+ *  Phantom displays are ignored.
  *
  *  @param did display identifier to search for
  *  @return #Display_Ref for the display, NULL if not found or
@@ -192,7 +202,7 @@ ddc_find_display_ref_by_criteria(Display_Criteria * criteria) {
 static Display_Ref *
 ddc_find_display_ref_by_display_identifier(Display_Identifier * did) {
    bool debug = false;
-   DBGTRC(debug, TRACE_GROUP, "Starting. did=%s", did_repr(did));
+   DBGTRC_STARTING(debug, TRACE_GROUP, "did=%s", did_repr(did));
    if (debug)
       dbgrpt_display_identifier(did, 1);
 
@@ -233,22 +243,22 @@ ddc_find_display_ref_by_display_identifier(Display_Identifier * did) {
 
    free(criteria);   // do not free pointers in criteria, they are owned by Display_Identifier
 
-   DBGTRC_RETURNING(debug, DDCA_TRC_NONE, dref_repr_t(result), "");
+   DBGTRC_RET_STRING(debug, DDCA_TRC_NONE, dref_repr_t(result), "");
    return result;
 }
 
 
 /** Searches the detected displays for one matching the criteria in a
- *  #Display_Identifier.
+ *  #Display_Identifier. Phantom displays are ignored.
  *
  *  @param pdid  pointer to a #Display_Identifier
  *  @param callopts  standard call options
  *  @return pointer to #Display_Ref for the display, NULL if not found
  *
  *  \todo
- *  If the criteria directly specify an access path
- *  (e.g. I2C bus number) and CALLOPT_FORCE specified, then create a
- *  temporary #Display_Ref, bypassing the list of detected monitors.
+ *  If the criteria directly specify an access path (e.g. I2C bus number) and
+ *  CALLOPT_FORCE is specified, then create a temporary #Display_Ref,
+ *  bypassing the list of detected monitors.
  */
 Display_Ref *
 get_display_ref_for_display_identifier(
