@@ -1,6 +1,6 @@
-/** \file ddc_read_capabilities.c */
+/** @file ddc_read_capabilities.c */
 
-// Copyright (C) 2014-2021 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2023 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #include <config.h>
@@ -22,9 +22,10 @@
 #include "base/ddc_errno.h"
 #include "base/rtti.h"
 #include "base/sleep.h"
+#include "base/status_code_mgt.h"
 #include "base/tuned_sleep.h"
 
-#ifdef USE_USB
+#ifdef ENABLE_USB
 #include "usb/usb_displays.h"
 #endif
 
@@ -35,8 +36,6 @@
 
 #include "ddc/ddc_read_capabilities.h"
 
-// Direct writes to stdout/stderr: none
-
 // Default trace class for this file
 static DDCA_Trace_Group TRACE_GROUP = DDCA_TRC_DDC;
 
@@ -64,8 +63,8 @@ get_capabilities_into_buffer(
    ddc_excp = multi_part_read_with_retry(
                dh,
                DDC_PACKET_TYPE_CAPABILITIES_REQUEST,
-               0x00,                       // no subtype for capabilities
-               false,                      // !all_zero_response_ok
+               0x00,                                 // no subtype for capabilities
+               Write_Read_Flag_Capabilities,         // special all zero response handling
                capabilities_buffer_loc);
    Buffer * cap_buffer = *capabilities_buffer_loc;
    ASSERT_IFF(cap_buffer, !ddc_excp);
@@ -93,7 +92,14 @@ get_capabilities_into_buffer(
 
 /** Gets the capabilities string for a display.
  *
- *  The value is cached as this is an expensive operation.
+ * For a display with an I2C device path:
+ *  - First, checks to see if it is already in the Display_Ref.
+ *  - If not, checks if the string is available in the capabilities cache.
+ *  - Finally, executes the VCP Get Capabilities command to obtain the
+ *    capabilities string.
+ *
+ *  For display with a USB device path
+ *  - Returns a synthesized capabilities string.
  *
  *  @param  dh       display handle
  *  @param  caps_loc location where to return pointer to capabilities string.
@@ -110,13 +116,13 @@ ddc_get_capabilities_string(
    bool debug = false;
    assert(dh);
    assert(dh->dref);
-   DBGTRC_STARTING(debug, TRACE_GROUP, "dh=%s", dh_repr(dh));
+   DBGTRC_STARTING(debug, TRACE_GROUP, "dh=%s, dh->dref->capabilities_string=|%s|",
+         dh_repr(dh), dh->dref->capabilities_string);
 
-   // Public_Status_Code psc = 0;
    Error_Info * ddc_excp = NULL;
    if (!dh->dref->capabilities_string) {
       if (dh->dref->io_path.io_mode == DDCA_IO_USB) {
-#ifdef USE_USB
+#ifdef ENABLE_USB
          // newly created string, can just  reference
          dh->dref->capabilities_string = usb_get_capabilities_string_by_dh(dh);
 #else
@@ -129,8 +135,9 @@ ddc_get_capabilities_string(
          DBGTRC_NOPREFIX(debug, TRACE_GROUP, "get_persistent_capabilities() returned |%s|",
                                     dh->dref->capabilities_string);
          if (dh->dref->capabilities_string && get_output_level() >= DDCA_OL_VERBOSE) {
-            char * s = get_capabilities_cache_file_name();
+            char * s = capabilities_cache_file_name();
             rpt_vstring(0, "Read cached capabilities string from %s", s);
+            SYSLOG2(DDCA_SYSLOG_INFO, "Read cached capabilities string from %s", s);
             free(s);
          }
 
@@ -138,7 +145,7 @@ ddc_get_capabilities_string(
             Buffer * pcaps_buffer;
             ddc_excp = get_capabilities_into_buffer(dh, &pcaps_buffer);
             if (!ddc_excp) {
-               dh->dref->capabilities_string = strdup((char *) pcaps_buffer->bytes);
+               dh->dref->capabilities_string = g_strdup((char *) pcaps_buffer->bytes);
                buffer_free(pcaps_buffer,__func__);
                set_persistent_capabilites(dh->dref->mmid, dh->dref->capabilities_string);
             }
