@@ -1,6 +1,6 @@
 /** @file data_structures.c  General purpose data structures */
 
-// Copyright (C) 2014-2023 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2025 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 /** \cond */
@@ -983,7 +983,10 @@ void test_value_array() {
 
 const Bit_Set_256 EMPTY_BIT_SET_256 = {{0}};
 
-/** Sets a flag in a #Bit_Set_256
+/** Sets a flag in a #Bit_Set_256.
+ *
+ *  The flag is set in the returned value.
+ *  Parm **flags** is NOT modified in place.
  *
  *  @param  flags   existing #Bit_Set_256 value
  *  @param  flagno  flag number to set (0 based)
@@ -1010,7 +1013,7 @@ Bit_Set_256 bs256_insert(
        char * bs2 = g_strdup(bs256_to_string_t(result, "",""));
        printf("(%s) old bitstring=%s, value %d, returning: %s\n",
               __func__, bs1, bitno, bs2);
-       free( bs1);
+       free(bs1);
        free(bs2);
     }
 
@@ -1018,6 +1021,35 @@ Bit_Set_256 bs256_insert(
 }
 
 
+/** Clears a flag in a #Bit_Set_256.
+ *
+ *  The flag is cleared in the returned value.
+ *  Parm **flags** is NOT modified in place.
+ *
+ *  @param  flags   existing #Bit_Set_256 value
+ *  @param  flagno  flag number to set (0 based)
+ *  @return updated set
+ */
+Bit_Set_256 bs256_remove(
+      Bit_Set_256 bitset,
+      Byte        bitno)
+{
+   bool debug = false;
+
+   Bit_Set_256 result = bitset;
+   int bytendx   = bitno >> 3;
+   int shiftct   = bitno & 0x07;
+   Byte flagbit  = 0x01 << shiftct;
+   if (debug) {
+      printf("(%s) bitno=0x%02x, bytendx=%d, shiftct=%d, flagbit=0x%02x\n",
+             __func__, bitno, bytendx, shiftct, flagbit);
+   }
+   result.bytes[bytendx] &= ~flagbit;
+
+   return result;
+}
+
+
 /** Tests if a bit is set in a #Bit_Set_256.
  *
  *  @param bitset  #Bit_Set_256 to check
@@ -1047,6 +1079,7 @@ bool bs256_contains(
     return result;
 }
 
+
 /** Returns the bit number of the first bit set.
  *  @param  bitset #Bit_Set_256 to check
  *  @return number of first bit that is set (0 based),
@@ -1114,15 +1147,15 @@ Bit_Set_256 bs256_and_not(
 }
 
 
-#define BB256_REPR_BUF_SZ (3*32+1)
+#define BS256_REPR_BUF_SZ (3*32+1)
 /** Represents a #Bit_Set_256 value as a sequence of 32 hex values.
  *
+ *  @param bbset value to represent
  *  @param buf   buffer in which to return value
  *  @param bufsz buffer size, must be at least #BB256_REPR_BUF_SZ
- *  @param bbset value to represent
  */
-void bb256_repr(char * buf, int bufsz, Bit_Set_256 bbset) {
-   assert(bufsz >= BB256_REPR_BUF_SZ);
+void bs256_repr(Bit_Set_256 bbset, char * buf, int bufsz) {
+   assert(bufsz >= BS256_REPR_BUF_SZ);
    g_snprintf(buf, bufsz,
               "%02x %02x %02x %02x %02x %02x %02x %02x "
               "%02x %02x %02x %02x %02x %02x %02x %02x "
@@ -1160,8 +1193,8 @@ int bs256_count(
       }
    }
    if (debug) {
-      char buf[BB256_REPR_BUF_SZ];
-      bb256_repr(buf, sizeof(buf), bbset);
+      char buf[BS256_REPR_BUF_SZ];
+      bs256_repr(bbset, buf, sizeof(buf));
       printf("(%s) Returning %d. bbset: %s\n", __func__, result, buf);
    }
    return result;
@@ -1194,11 +1227,11 @@ int bs256_count(
  *  The value returned is valid until the next call to this function in the
  *  current thread.
  *
- *  @param  bitset value to represent
+ *  @param  bitset          value to represent
  *  @param  decimal_format  if true,  represent values as decimal numbers
  *                          if false, represent values as hex numbers
- *  @param  value_prefix  prefix for each hex number, typically "0x" or ""
- *  @param  sepstr        string to insert between each value, typically "", ",", or " "
+ *  @param  value_prefix    prefix for each hex number, typically "0x" or ""
+ *  @param  sepstr          string to insert between each value, typically "", ",", or " "
  *  @return string representation, caller should not free
  */
 static char *
@@ -1285,7 +1318,7 @@ bs256_to_string_t(
  *  The value returned is valid until the next call to this function or to
  *  #bs256_to_string() in the current thread.
  *
- *  @param  bitset value to represent
+ *  @param  bitset        value to represent
  *  @param  value_prefix  prefix for each decimal number
  *  @param  sepstr        string to insert between each value, typically "", ",", or " "
  *  @return string representation, caller should not free
@@ -1539,15 +1572,15 @@ const Bit_Set_32 EMPTY_BIT_SET_32 = 0;
 const int BIT_SET_32_MAX = 32;
 
 
-#define BB32_REPR_BUF_SZ 10
+#define BS32_REPR_BUF_SZ 10
 /** Represents a #Bit_Set_32 value as a sequence of 4 hex values.
  *
  *  @param buf   buffer in which to return value
  *  @param bufsz buffer size, must be at least #BB256_REPR_BUF_SZ
  *  @param bbset value to represent
  */
-void bb32_repr(char * buf, int bufsz, Bit_Set_32 bbset) {
-   assert(bufsz >= BB256_REPR_BUF_SZ);
+void bs32_repr(Bit_Set_32 bbset, char * buf, int bufsz) {
+   assert(bufsz >= BS32_REPR_BUF_SZ);
    g_snprintf(buf, bufsz, "0x%08x", bbset);
 }
 
@@ -1569,8 +1602,8 @@ int bs32_count(
          result += 1;
    }
    if (debug) {
-      char buf[BB32_REPR_BUF_SZ];
-      bb32_repr(buf, sizeof(buf), bbset);
+      char buf[BS32_REPR_BUF_SZ];
+      bs32_repr(bbset, buf, sizeof(buf));
       printf("(%s) Returning %d. bbset: %s\n", __func__, result, buf);
    }
    return result;
@@ -1874,14 +1907,16 @@ bool bs256_store_bytehex_list(Bit_Set_256 * pbitset, char * start, int len) {
 
 /** Adds function to a set of registered callbacks
  *
- * @param  array of registered callbacks
+ * @param  registered_callback_loc  address of ptr to GPtrArray of registered callbacks
  * @param  function to add
- * @retval true  success
+ *
+ * If **registered_callcack_loc** is null on entry, a new GPtrArray is allocated and
+ * its address is returned in **registered_callback_loc**.
  *
  * @remark
  * It is not an error if the function is already registered.
  */
-bool generic_register_callback(GPtrArray** registered_callbacks_loc, void * func) {
+void generic_register_callback(GPtrArray** registered_callbacks_loc, void * func) {
    bool debug = false;
    DBGF(debug, "Starting. registered_callbacks=%p, func=%p", *registered_callbacks_loc, func);
 
@@ -1900,21 +1935,22 @@ bool generic_register_callback(GPtrArray** registered_callbacks_loc, void * func
       g_ptr_array_add(*registered_callbacks_loc, func);
    }
 
-   bool result = true;
-   DBGF(debug, "Done.     Returning %s", SBOOL(result));
-   return result;
+   // DBGF(debug, "Done.     Returning %s", SBOOL(new_registration));
+   // return new_registration;
+   DBGF(debug, "Done.     new_registration = %s", SBOOL(new_registration));
 }
 
 
 /** Unregisters a callback function
  *
- *  @param func function to remove
+ *  @param  func  function to remove
  *  @retval true  function deregistered
  *  @retval false function not found
  *   */
 bool generic_unregister_callback(GPtrArray* registered_callbacks, void *func) {
      bool debug = false;
      DBGF(debug, "Starting. registered_callbacks=%p, func=%p", registered_callbacks, func);
+
      bool found = false;
      if (registered_callbacks) {
         for (int ndx = 0; ndx < registered_callbacks->len; ndx++) {
@@ -1924,6 +1960,7 @@ bool generic_unregister_callback(GPtrArray* registered_callbacks, void *func) {
            }
         }
      }
+
      DBGF(debug, "Done.     Returning: %s", SBOOL(found));
      return found;
 }
