@@ -1,4 +1,4 @@
-/** @f error_info.c
+/** \f error_info.c
  *
  *  Struct for reporting errors.
  *
@@ -9,13 +9,12 @@
  *  error is retained for use by higher levels in the call stack.
  */
 
-// Copyright (C) 2017-2023 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2017-2024 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 
 /** \cond */
 
-// #define _GNU_SOURCE     // for reallocarray() in stdlib.h
 #include <assert.h>
 #include <glib-2.0/glib.h>
 #include <stdlib.h>
@@ -24,8 +23,10 @@
 
 #include "debug_util.h"
 #include "glib_util.h"
+#include "msg_util.h"
 #include "report_util.h"
 #include "string_util.h"
+#include "traced_function_stack.h"
 
 #include "error_info.h"
 
@@ -33,6 +34,11 @@
 // Validates a pointer to an #Error_Info, using asserts
 #define VALID_ERROR_INFO_PTR(ptr) \
    assert(ptr); \
+   if (memcmp(ptr->marker, ERROR_INFO_MARKER, 4) != 0) { \
+      DBG("Invalid ptr->marker, ptr=%p", ptr); \
+      show_backtrace(1); \
+      debug_current_traced_function_stack(false); \
+   } \
    assert(memcmp(ptr->marker, ERROR_INFO_MARKER, 4) == 0);
 
 
@@ -91,6 +97,7 @@ errinfo_all_causes_same_status(
 {
    bool debug = false;
    DBGF(debug, "Starting. status_code=%d, erec=%s", status_code, errinfo_summary(erec));
+
    bool all_same = false;
    if (erec) {
       VALID_ERROR_INFO_PTR(erec);
@@ -106,10 +113,25 @@ errinfo_all_causes_same_status(
          }
       }
    }
+
    DBGF(debug, "Returning: %s", SBOOL(all_same));
    return all_same;
 }
 
+#ifdef ALT
+bool
+errinfo_all_causes_same_status(Error_Info * ddc_excp, int status_code) {
+     bool all_same = true;
+     for (int ndx = 0; ndx < ddc_excp->cause_ct; ndx++) {
+        if (ddc_excp->causes[ndx]->status_code != status_code) {
+           all_same = false;
+           break;
+        }
+     }
+     return all_same;
+  }
+#endif
+
 
 //
 // Instance destruction
@@ -117,22 +139,21 @@ errinfo_all_causes_same_status(
 
 /** Releases a #Error_Info instance, including all instances it points to.
  *
- *  \param erec pointer to #Error_Info instance,
- *              do nothing if NULL
+ *  \param erec pointer to #Error_Info instance, do nothing if NULL
  */
 void
 errinfo_free(Error_Info * erec){
    bool debug = false;
    if (debug) {
-      printf("(%s) Starting. erec=%p\n", __func__, (void*)erec);
-      show_backtrace(2);
-      errinfo_report(erec, 2);
+      DBG("Starting. erec=%p", (void*)erec);
+      // show_backtrace(2);
+      // errinfo_report(erec, 2);
    }
    if (erec) {
       VALID_ERROR_INFO_PTR(erec);
 
       if (debug) {
-         printf("(%s) Freeing exception: \n", __func__);
+         DBG("Freeing exception:");
          errinfo_report(erec, 2);
       }
 
@@ -140,9 +161,11 @@ errinfo_free(Error_Info * erec){
          free(erec->detail);
 
       if (erec->cause_ct > 0) {
+         DBGF(debug, "Freeing causes...");
          for (int ndx = 0; ndx < erec->cause_ct; ndx++) {
             errinfo_free(erec->causes[ndx]);
          }
+         DBGF(debug, "Freeing erec->causes = %p", erec->causes);
          free(erec->causes);
       }
 #ifdef ALT
@@ -157,8 +180,8 @@ errinfo_free(Error_Info * erec){
       erec->marker[3] = 'x';
       free(erec);
    }
-   if (debug)
-      printf("(%s) Done.\n", __func__);
+
+   DBGF(debug, "Done.  Free'd: %p", (void*) erec);
 }
 
 
@@ -212,6 +235,14 @@ errinfo_set_status(Error_Info * erec, int code) {
 }
 
 
+/** Sets the detail string in a existing #Error_Info instance.
+ *  If there is already a detail string in the instance, it is replaced.
+ *  The substitution values for the detail string are specified as a va_list.
+ *
+ *  \param  erec   pointer to instance
+ *  \param  detail detail format string
+ *  \param  args   arguments for detail string
+ */
 static void
 errinfo_set_detailv(
       Error_Info * erec,
@@ -229,7 +260,7 @@ errinfo_set_detailv(
  *
  *  \param  erec   pointer to instance
  *  \param  detail detail format string
- *  \param  ...    arguments for format string
+ *  \param  ...    arguments for detail format string
  */
 void
 errinfo_set_detail(
@@ -250,6 +281,39 @@ errinfo_set_detail(
 }
 
 
+/** Make a deep copy of a #Error_Info record.
+ *
+ *  @param  old  record to copy
+ *  @return copy of record
+ */
+Error_Info * errinfo_copy(Error_Info* old) {
+   bool debug = false;
+   DBGF(debug, "Starting. old=%p", (void*) old);
+
+   Error_Info * new = calloc(1, sizeof(Error_Info));
+   memcpy(new->marker, old->marker, 4);
+   new->status_code = old->status_code;
+   if (old->func)
+    new->func = g_strdup(old->func);
+   if (old->detail)
+      new->detail = g_strdup(old->detail);
+   new->max_causes = old->max_causes;
+   new->cause_ct = old->cause_ct;
+   if (new->cause_ct == 0)
+      new->causes = empty_list;
+   else {
+      new->causes = calloc(new->max_causes+1, sizeof(Error_Info*));
+      DBGF(debug, "Allocated new->causes=%p, new->cause_ct=%d", (void*) new->causes, new->cause_ct);
+   }
+   for (int ndx = 0; ndx < new->cause_ct; ndx++) {
+      new->causes[ndx] = errinfo_copy(old->causes[ndx]);
+   }
+
+   DBGF(debug, "Done.  Returning %p", (void*) new);
+   return new;
+}
+
+
 /** Adds a cause to an existing #Error_Info instance
  *
  *  \param  parent instance to which cause will be added
@@ -260,12 +324,13 @@ errinfo_add_cause(
       Error_Info * parent,
       Error_Info * cause)
 {
-   // printf("(%s) cause=%p\n", __func__, cause);
+   bool debug = false;
+   DBGF(debug, "parent=%p, cause=%p", parent, cause);
    VALID_ERROR_INFO_PTR(parent);
    VALID_ERROR_INFO_PTR(cause);
+   DBGF(debug, "parent->cause_ct = %d, parent->max_causes = %d",
+               parent->cause_ct, parent->max_causes);
 
-   // printf("(%s) parent->cause_ct = %d, parent->max_causes = %d\n",
-   //         __func__, parent->cause_ct, parent->max_causes);
    if (parent->cause_ct == parent->max_causes) {
       int new_max = parent->max_causes + CAUSE_ALLOC_INCREMENT;
 #ifdef ALT
@@ -275,11 +340,10 @@ errinfo_add_cause(
       parent->causes = new_causes;
 #endif
       if (parent->causes == empty_list) {
-         // printf("(%s) empty_list\n", __func__);
+         DBGF(debug, "empty_list");
          parent->causes = calloc(new_max+1, sizeof(Error_Info *) );
       }
       else {
-         // printf("(%s) realloc\n", __func__);
          // works, but requires _GNU_SOURCE feature test macro:
          // parent->causes = reallocarray(parent->causes, new_max+1, sizeof(Error_Info*) );
          void * new_causes = calloc(new_max+1, sizeof(Error_Info*) );
@@ -289,9 +353,9 @@ errinfo_add_cause(
       }
       parent->max_causes = new_max;
    }
-   // printf("(%s) parent->causes = %p\n", __func__, parent->causes);
-   // printf("(%s) cause_ct=%d\n", __func__, parent->cause_ct);
-   // printf("(%s) %p", __func__, &parent->causes[parent->cause_ct]);
+
+   DBGF(debug, "causes = %p, cause_ct=%d", parent->causes, parent->cause_ct);
+   // DBGF(debug, "%p", &parent->causes[parent->cause_ct]);
 
    parent->causes[parent->cause_ct++] = cause;
 
@@ -302,6 +366,8 @@ errinfo_add_cause(
    }
    g_ptr_array_add(parent->causes_alt, cause);
 #endif
+
+   DBGF(debug, "Done. causes = %p, cause_ct=%d", parent->causes, parent->cause_ct);
 }
 
 
@@ -311,7 +377,7 @@ errinfo_add_cause(
 
 /** Creates a new #Error_Info instance with the specified status code,
  *  function name, and detail string.  The substitution values for the
- *  detail string are specified as an arg_list.
+ *  detail string are specified as a va_list.
  *
  *  \param  status_code  status code
  *  \param  func         name of function generating status code
@@ -326,6 +392,9 @@ errinfo_newv(
       const char *   detail,
       va_list        args)
 {
+   bool debug = false;
+   DBGF(debug, "Starting. status_code=%d, func=%s, detail=%s", status_code, func, detail);
+
    Error_Info * erec = calloc(1, sizeof(Error_Info));
    memcpy(erec->marker, ERROR_INFO_MARKER, 4);
    erec->status_code = status_code;
@@ -335,6 +404,8 @@ errinfo_newv(
    if (detail) {
       erec->detail = g_strdup_vprintf(detail, args);
    }
+
+   DBGF(debug, "Done:    Returning %p", erec);
    return erec;
 }
 
@@ -365,6 +436,17 @@ errinfo_new(
 }
 
 
+/** Creates a new #Error_Info instance with a detail string, including a
+ * reference to another instance that is the cause of the current error.
+ *  The substitution values for the detail string are specified as a va_list.
+ *
+ *  \param  status_code  status code
+ *  \param  cause        pointer to another #Error_Info that is included as a cause
+ *  \param  func         name of function creating new instance
+ *  \param  detail_fmt   optional detail format string
+ *  \param  args         substitution value for detail_fmt
+ *  \return pointer to new instance
+ */
 static Error_Info *
 errinfo_new_with_causev(
       int            status_code,
@@ -379,6 +461,7 @@ errinfo_new_with_causev(
    return erec;
 }
 
+
 /** Creates a new #Error_Info instance with a detail string, including a
  * reference to another instance that is the cause of the current error.
  *
@@ -457,13 +540,10 @@ errinfo_new_with_causes(
 }
 
 
-/** Creates a new #Error_Info instance with a collection of
- *  instances specified as the causes. The collection is
- *  passed as a GPtrArray.
- *
- *  Note that the pointers in the **causes** array are copied to the new
- *  #Error_Info instance. The pointers should not be freed when destroying
- *  the original array of causes.
+/** Creates a new #Error_Info instance with a collection of instances specified
+ *  as the causes. The collection is passed as a GPtrArray.  A deep copy is
+ *  made of each instance in the collection, and the causes collection
+ *  is unchanged.
  *
  *  \param  code            status code of the new instance
  *  \param  causes          GPtrArray of #Error_Info instances
@@ -479,13 +559,18 @@ Error_Info * errinfo_new_with_causes_gptr(
       char *         detail,
       ...)
 {
+   bool debug = false;
+   DBGF(debug, "Starting.  status_code-%d, detail=%s", status_code, detail);
+
    va_list ap;
    va_start(ap, detail);
    Error_Info * result = errinfo_newv(status_code, func, detail, ap);
    va_end(ap);
    for (int ndx = 0; ndx < causes->len; ndx++) {
-      errinfo_add_cause(result, g_ptr_array_index(causes,ndx));
+         errinfo_add_cause(result, errinfo_copy(g_ptr_array_index(causes,ndx)));
    }
+
+   DBGF(debug, "Returning: %p", (void*) result);
    return result;
 }
 
@@ -551,43 +636,52 @@ default_status_code_desc(int rc) {
 }
 
 
+/** Appends a comma separated string of the status code names of the
+ *  causes in an array of #Error_Info to an existing string.
+ *  Multiple consecutive identical names are replaced with a
+ *  single name and a parenthesized instance count.
+ *
+ *  \param  erec     pointer to array of pointers to #Error_Info instances
+ *  \param  error_ct number of errors
+ *  \return modified comma separated string
+ */
 static GString *
 errinfo_array_summary_gs(
-      struct error_info **  errors,    ///<  pointer to array of pointers to Error_Info
+      struct error_info **  errors,    ///<  pointer to array of pointers to #Error_Info
       int                   error_ct,  ///<  number of causal errors
       GString *             gs)        ///<  append result here
 {
-      bool first = true;
+   bool first = true;
 
-      int ndx = 0;
-      while (ndx < error_ct) {
-         // printf("(%s) this error = %p\n", __func__, errors[ndx]);
-         int this_psc = errors[ndx]->status_code;
-         int cur_ct = 1;
+   int ndx = 0;
+   while (ndx < error_ct) {
+      // printf("(%s) this error = %p\n", __func__, errors[ndx]);
+      int this_psc = errors[ndx]->status_code;
+      int cur_ct = 1;
 
-         for (int i = ndx+1; i < error_ct; i++) {
-            if (errors[i]->status_code != this_psc)
-               break;
-            cur_ct++;
-         }
-
-         if (first)
-            first = false;
-         else
-            g_string_append(gs, ", ");
-         if (errinfo_name_func)
-            g_string_append(gs, errinfo_name_func(this_psc));
-         else {
-            char buf[20];
-            snprintf(buf, 20, "%d",this_psc);
-            buf[19] = '\0';
-            g_string_append(gs, buf);
-         }
-         if (cur_ct > 1)
-            g_string_append_printf(gs, "(%d)", cur_ct);
-         ndx += cur_ct;
+      for (int i = ndx+1; i < error_ct; i++) {
+         if (errors[i]->status_code != this_psc)
+            break;
+         cur_ct++;
       }
 
+      if (first)
+         first = false;
+      else
+         g_string_append(gs, ", ");
+      if (errinfo_name_func)
+         g_string_append(gs, errinfo_name_func(this_psc));
+      else {
+         char buf[20];
+         snprintf(buf, 20, "%d",this_psc);
+         buf[19] = '\0';
+         g_string_append(gs, buf);
+      }
+      if (cur_ct > 1)
+         g_string_append_printf(gs, "(%d)", cur_ct);
+      ndx += cur_ct;
+   }
+
    return gs;
 }
 
@@ -597,7 +691,8 @@ errinfo_array_summary_gs(
  *  Multiple consecutive identical names are replaced with a
  *  single name and a parenthesized instance count.
  *
- *  \param  erec  pointer to array of #Error_Info instances
+ *  \param  errors   pointer to array of pointers to #Error_Info instances
+ *  \param  error_ct number of instances
  *  \return comma separated string, caller is responsible for freeing
  */
 char *
@@ -742,12 +837,13 @@ errinfo_causes_string(Error_Info * erec) {
 #endif
 
 
-
-
-/** Emits a full report of the contents of the specified #Error_Info,
- *  using report functions.
+/** Creates a full report of the contents of the specified #Error_Info,
+ *  using report functions. if **collector** is non-null, the lines of
+ *  the report are appended to the array.  If it is null, the lines
+ *  are written the current report output destination.
  *
  *  \param  erec   pointer to #Error_Info
+ *  \param  collector collects lines of the report
  *  \param  depth  logical indentation depth
  */
 void
@@ -786,25 +882,24 @@ errinfo_report_collect(Error_Info * erec, GPtrArray* collector, int depth) {
 }
 
 
+/** Emits a full report of the contents of the specified #Error_Info,
+ *  using report functions.
+ *
+ *  \param  erec   pointer to #Error_Info
+ *  \param  depth  logical indentation depth
+ */
 void
 errinfo_report(Error_Info * erec, int depth) {
       errinfo_report_collect(erec, NULL, depth);
 }
 
 
-
-#ifdef NO
-void errinfo_free_with_report(
-      Error_Info *  erec,
-      bool          report,
-      const char *  func)
-{
-   (void) errinfo_free_with_report_collect(erec, report, func, false);
-}
-#endif
-
-
-
+/** Reports detail strings for a #Error_Info record and
+ *  each of its contained errors.
+ *
+ *  \param erec  pointer = #Error_Info instance
+ *  \param depth logical indentation depth
+ */
 void
 errinfo_report_details(Error_Info * erec, int depth) {
    assert(erec);
@@ -833,7 +928,7 @@ errinfo_report_details(Error_Info * erec, int depth) {
  *  The returned value is valid until the next call to this function in the
  *  current thread, and should not be freed by the caller.
  *
- *  \param erec  pointer to #Error_Info instance
+ *  \param  erec  pointer to #Error_Info instance
  *  \return string summary of error
  */
 char *
@@ -857,7 +952,7 @@ errinfo_summary(Error_Info * erec) {
       buf1 = g_strdup_printf("Error_Info[%s in %s]", desc, erec->func);
    }
    else {
-      char * causes   = errinfo_causes_string(erec);
+      char * causes = errinfo_causes_string(erec);
       buf1 = g_strdup_printf("Error_Info[%s in %s, causes: %s]", desc, erec->func, causes);
       free(causes);
    }
