@@ -3,12 +3,16 @@
  *  Report utility package
  */
 
-// Copyright (C) 2014-2022 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2025 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #ifndef REPORT_UTIL_H_
 #define REPORT_UTIL_H_
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /** \cond */
 #include <glib-2.0/glib.h>
 #include <stdio.h>
@@ -18,6 +22,26 @@
 #include "coredefs_base.h"
 #include "string_util.h"
 
+#ifdef PERHAPS
+#define RPT_PREFIX_NONE   0x00
+#define RPT_PREFIX_PID    0x01
+#define RPT_PREFIX_TID    0x02
+#define RPT_PREFIX_TS     0x04
+#define RPT_PREFIX_WTD    0x08
+
+extern __thread Byte rpt_prefix_options;
+#endif
+
+// If set, report messages are written to the system log, not the terminal
+extern bool redirect_reports_to_syslog;
+// If set, report messages have a letter tag appended, as a debugging aid
+// for identifying message source location.
+extern bool tag_output;
+
+void rpt_set_default_ornamentation_enabled(bool onoff);
+bool rpt_get_ornamentation_enabled();
+bool rpt_set_ornamentation_enabled(bool enabled);
+
 void rpt_set_default_output_dest(FILE* output_dest);
 void rpt_push_output_dest(FILE* new_dest);
 void rpt_pop_output_dest();
@@ -33,10 +57,12 @@ void rpt_flush();
 void rpt_nl();
 void rpt_title(const char * title, int depth);
 void rpt_label(int depth, const char * text);
+void rpt_label_collect(int depth, GPtrArray* collector, const char * text);
 void rpt_multiline(int depth, ...);
 void rpt_g_ptr_array(int depth, GPtrArray * strings);
 
 void rpt_vstring(int depth, char * format, ...) ;
+void rpt_vstring_collect(int depth, GPtrArray* collector, char * format, ...);
 void rpt_2col(char * s1,  char * s2,  int col2offset, bool offset_absolute, int depth);
 void rpt_structure_loc(const char * name, const void * ptr, int depth);
 void rpt_hex_dump(const Byte * data, int size, int depth);
@@ -83,4 +109,8 @@ void rpt_ifval2(char * name,
                 Flag_Dictionary * pDict,
                 int    depth);
 
+#ifdef __cplusplus
+}    // extern "C"
+#endif
+
 #endif /* REPORT_UTIL_H_ */
