@@ -70,6 +70,20 @@ rpt_attr_text(
 #define GET_ATTR_TEXT(value_loc, fn_segment, ...) \
    rpt_attr_text(-1, value_loc, fn_segment, ##__VA_ARGS__, NULL)
 
+bool
+rpt_attr_int(
+      int          depth,
+      int *        value_loc,
+      const char * fn_segment,
+      ...);
+
+#define RPT_ATTR_INT(depth, value_loc, fn_segment, ...) \
+   rpt_attr_int(depth, value_loc, fn_segment, ##__VA_ARGS__, NULL)
+
+#define GET_ATTR_INT(value_loc, fn_segment, ...) \
+   rpt_attr_int(-1, value_loc, fn_segment, ##__VA_ARGS__, NULL)
+
+
 bool
 rpt_attr_binary(
       int           depth,
@@ -134,13 +148,13 @@ rpt_attr_single_subdir(
    rpt_attr_single_subdir(-1, value_loc, predicate_func, predicate_val, fn_segment, ##__VA_ARGS__, NULL)
 
 bool
-rpt_attr_note_indirect_subdir(
+rpt_attr_note_subdir(
       int          depth,
       char **      value_loc,
       const char * fn_segment,
       ...);
 
-#define RPT_ATTR_NOTE_INDIRECT_SUBDIR(depth, value_loc, fn_segment, ...) \
-   rpt_attr_note_indirect_subdir(depth, value_loc, fn_segment,  ##__VA_ARGS__, NULL)
+#define RPT_ATTR_NOTE_SUBDIR(depth, value_loc, fn_segment, ...) \
+   rpt_attr_note_subdir(depth, value_loc, fn_segment,  ##__VA_ARGS__, NULL)
 
 #endif /* SYSFS_UTIL_H_ */
